﻿using System;
using System.IO;

namespace HeuristicLab.Problems.QuadraticAssignment {
  public class QAPLIBSolutionParser {
    public int Size { get; private set; }
    public int[] Assignment { get; private set; }
    public double Qualiy { get; private set; }
    public Exception Error { get; private set; }

    public QAPLIBSolutionParser() {
      Reset();
    }

    public void Reset() {
      Size = 0;
      Assignment = null;
      Qualiy = double.NaN;
      Error = null;
    }

    public bool Parse(string file) {
      using (Stream stream = new FileStream(file, FileMode.Open, FileAccess.Read)) {
        return Parse(stream);
      }
    }

    /// <summary>
    /// Reads from the given stream data which is expected to be in the QAPLIB format.
    /// </summary>
    /// <remarks>
    /// The stream is not closed or disposed. The caller has to take care of that.
    /// </remarks>
    /// <param name="stream">The stream to read data from.</param>
    /// <returns>True if the file was successfully read or false otherwise.</returns>
    public bool Parse(Stream stream) {
      Error = null;
      try {
        StreamReader reader = new StreamReader(stream);
        char[] delim = new char[] { ' ', ',' }; // comma is added for nug30.sln which is the only file that separates the permutation with commas
        string[] firstline = reader.ReadLine().Split(delim, StringSplitOptions.RemoveEmptyEntries);
        Size = int.Parse(firstline[0]);
        Qualiy = double.Parse(firstline[1]);
        Assignment = new int[Size];
        int read = 0;
        while (!reader.EndOfStream) {
          string valLine = reader.ReadLine();
          string[] vals = valLine.Split(delim, StringSplitOptions.RemoveEmptyEntries);
          for (int j = 0; j < vals.Length; j++) {
            Assignment[read++] = int.Parse(vals[j]);
          }
        }
        return true;
      } catch (Exception e) {
        Error = e;
        return false;
      }
    }
  }
}
